@@ -1,4 +1,4 @@
-/***************************************************************************
+/**************************************************************************
  *
  * library.c
  *
@@ -11,8 +11,6 @@
  * This file contains all the "glue code" required to use the canon
  * driver with libgphoto2.
  *
- * $Id: library.c 9256 2006-10-03 19:33:16Z hun $
- *
  ****************************************************************************/
 
 
@@ -94,16 +92,14 @@ extern long int timezone;
 #define UPLOAD_BOOL FALSE
 #endif
 
-const struct canonCaptureSizeClassStruct captureSizeArray[] = {
+static const struct canonCaptureSizeClassStruct captureSizeArray[] = {
         {CAPTURE_COMPATIBILITY, N_("Compatibility mode")},
         {CAPTURE_THUMB, N_("Thumbnail")},
         {CAPTURE_FULL_IMAGE, N_("Full Image")},
 	{0, NULL}
 };
 
-#ifdef CANON_EXPERIMENTAL_20D
-
-const struct canonIsoStateStruct isoStateArray[] = {
+static const struct canonIsoStateStruct isoStateArray[] = {
         {ISO_100, "ISO 100"},
         {ISO_125, "ISO 125"},
         {ISO_160, "ISO 160"},
@@ -120,7 +116,7 @@ const struct canonIsoStateStruct isoStateArray[] = {
 	{0, NULL},
 };
 
-const struct canonShutterSpeedStateStruct shutterSpeedStateArray[] = {
+static const struct canonShutterSpeedStateStruct shutterSpeedStateArray[] = {
         {SHUTTER_SPEED_BULB, "Bulb"},
         {SHUTTER_SPEED_30_SEC, "30 seconds"},
         {SHUTTER_SPEED_25_SEC, "25 seconds"},
@@ -177,7 +173,7 @@ const struct canonShutterSpeedStateStruct shutterSpeedStateArray[] = {
 	{0, NULL},
 };
 
-const struct canonApertureStateStruct apertureStateArray[] = {
+static const struct canonApertureStateStruct apertureStateArray[] = {
         {APERTURE_F2_8, "f2.8"},
         {APERTURE_F3_2, "f3.2"},
         {APERTURE_F3_5, "f3.5"},
@@ -200,7 +196,7 @@ const struct canonApertureStateStruct apertureStateArray[] = {
 	{0, NULL},
 };
 
-const struct canonFocusModeStateStruct focusModeStateArray[] = {
+static const struct canonFocusModeStateStruct focusModeStateArray[] = {
         {AUTO_FOCUS_ONE_SHOT, N_("Auto focus: one-shot")},
         {AUTO_FOCUS_AI_SERVO, N_("Auto focus: AI servo")},
         {AUTO_FOCUS_AI_FOCUS, N_("Auto focus: AI focus")},
@@ -208,7 +204,7 @@ const struct canonFocusModeStateStruct focusModeStateArray[] = {
 	{0, NULL},
 };
 
-const struct canonResolutionStateStruct resolutionStateArray[] = {
+static const struct canonResolutionStateStruct resolutionStateArray[] = {
 	{RESOLUTION_RAW, N_("RAW"),
 	 0x04, 0x02, 0x00},
 	{RESOLUTION_SMALL_NORMAL_JPEG, N_("Small Normal JPEG"),
@@ -238,8 +234,6 @@ const struct canonResolutionStateStruct resolutionStateArray[] = {
 	{0, NULL, 0, 0, 0},
 };
 
-#endif /* CANON_EXPERIMENTAL_20D */
-
 /**
  * camera_id:
  * @id: string buffer to receive text identifying camera type
@@ -1000,6 +994,44 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 	return GP_OK;
 }
 
+static int
+storage_info_func (
+	CameraFilesystem *fs,
+	CameraStorageInformation **sinfos, int *nrofsinfos,
+	void *data, GPContext *context
+) {
+	char root[10];
+	Camera *camera = (Camera*)data;
+
+	if (!check_readiness (camera, context))
+		return GP_ERROR_IO;
+
+	camera->pl->cached_drive = canon_int_get_disk_name (camera, context);
+	if (!camera->pl->cached_drive) {
+		gp_context_error (context, _("Could not get disk name: %s"),
+				  _("No reason available"));
+		return GP_ERROR_IO;
+	}
+	snprintf (root, sizeof (root), "%s\\", camera->pl->cached_drive);
+	canon_int_get_disk_name_info (camera, root, &camera->pl->cached_capacity,
+				    &camera->pl->cached_available, context);
+
+	*sinfos = (CameraStorageInformation*) calloc (sizeof (CameraStorageInformation), 1);
+	*nrofsinfos = 1;
+	(*sinfos)->fields = GP_STORAGEINFO_BASE;
+	strcpy ((*sinfos)->basedir, "/");
+	if (camera->pl->cached_drive) {
+		(*sinfos)->fields = GP_STORAGEINFO_LABEL;
+		strcpy ((*sinfos)->basedir, camera->pl->cached_drive);
+	}
+	(*sinfos)->fields |= GP_STORAGEINFO_MAXCAPACITY;
+	(*sinfos)->capacitykbytes = camera->pl->cached_capacity / 1024;
+	(*sinfos)->fields |= GP_STORAGEINFO_FREESPACEKBYTES;
+	(*sinfos)->freekbytes = camera->pl->cached_available / 1024;
+	return GP_OK;
+}
+	
+
 /****************************************************************************/
 
 static int
@@ -1427,11 +1459,8 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	CameraWidget *t, *section;
 	char power_str[128], firm[64];
 
-#ifdef CANON_EXPERIMENTAL_20D
 	int iso, shutter_speed, aperture, focus_mode;
 	int res_byte1, res_byte2, res_byte3;
-#endif
-
 	int pwr_status, pwr_source, res, i, menuval;
 	time_t camtime;
 
@@ -1480,8 +1509,6 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_append (section, t);
 
 
-#ifdef CANON_EXPERIMENTAL_20D
-
 	/********************* Release params **********************/
 
 	/* Necessary for release params: ISO, aperture, etc. */
@@ -1667,9 +1694,6 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 
 	/************************ end release params ************************/
 
-#endif /* CANON_EXPERIMENTAL_20D */
-
-
 	gp_widget_new (GP_WIDGET_TEXT, _("Date and Time (readonly)"), &t);
 	gp_widget_set_name (t, "datetime");
 	if (camera->pl->cached_ready == 1) {
@@ -1734,13 +1758,11 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_append (section, t);
 #endif /* CANON_EXPERIMENTAL_UPLOAD */
 
-#ifdef CANON_EXPERIMENTAL_20D
 	if (camera->pl->remote_control) {
 		res = canon_int_end_remote_control (camera, context);
 		if (res != GP_OK)   
 			return GP_ERROR;
 	};
-#endif
 
 	return GP_OK;
 }
@@ -1752,12 +1774,8 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	CameraWidget *w;
 	char *wvalue;
 	int i, val;
-#ifdef CANON_EXPERIMENTAL_20D
-#ifdef CANON_EXPERIMENTAL_SET_RELEASE_PARAMS
 	int res;
 	unsigned char iso, shutter_speed, aperture, focus_mode;
-#endif
-#endif
 	char str[16];
 
 	GP_DEBUG ("camera_set_config()");
@@ -1794,29 +1812,6 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 
 	}
 
-#ifdef CANON_EXPERIMENTAL_20D
-# ifndef CANON_EXPERIMENTAL_SET_RELEASE_PARAMS
-
-	/* Return errors if the user attempts to change any of the release
-	   parameters. */
-	gp_widget_get_child_by_label (window, _("Shutter speed"), &w);
-	if (gp_widget_changed (w)) 
-		return GP_ERROR;
-	gp_widget_get_child_by_label (window, _("ISO speed"), &w);
-	if (gp_widget_changed (w)) 
-		return GP_ERROR;
-	gp_widget_get_child_by_label (window, _("Aperture"), &w);
-	if (gp_widget_changed (w)) 
-		return GP_ERROR;
-	gp_widget_get_child_by_label (window, _("Resolution"), &w);
-	if (gp_widget_changed (w)) 
-		return GP_ERROR;
-	gp_widget_get_child_by_label (window, _("Focus mode"), &w);
-	if (gp_widget_changed (w)) 
-		return GP_ERROR;
-
-# else 
-
 	gp_widget_get_child_by_label (window, _("ISO speed"), &w);
 	if (gp_widget_changed (w)) {
 		gp_widget_get_value (w, &wvalue);
@@ -2017,9 +2012,6 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 		}
 	}
 
-# endif /* CANON_EXPERIMENTAL_SET_RELEASE_PARAMS */
-#endif /* CANON_EXPERIMENTAL_20D */
-
 	gp_widget_get_child_by_label (window, _("Set camera date to PC date"), &w);
 	if (gp_widget_changed (w)) {
 		gp_widget_get_value (w, &wvalue);
@@ -2199,7 +2191,8 @@ static CameraFilesystemFuncs fsfuncs = {
 	.del_file_func = delete_file_func,
 	.put_file_func = put_file_func,
 	.make_dir_func = make_dir_func,
-	.remove_dir_func = remove_dir_func
+	.remove_dir_func = remove_dir_func,
+	.storage_info_func = storage_info_func
 };
 
 int
