@@ -152,6 +152,7 @@ gp_port_library_list (GPPortInfoList *list)
 	bus = usb_get_busses();
 	while (bus) {
 		for (dev = bus->devices; dev; dev = dev->next) {
+			char *s;
 			/* Devices which are definitely not cameras. */
 			if (	(dev->descriptor.bDeviceClass == USB_CLASS_HUB)		||
 				(dev->descriptor.bDeviceClass == USB_CLASS_HID)		||
@@ -184,6 +185,8 @@ gp_port_library_list (GPPortInfoList *list)
 			 * and the Ricoh erronously reports it.
 			 */ 
 			sprintf (info.path, "usb:%s,%s", bus->dirname, dev->filename);
+			/* On MacOS X we might get usb:006,002-04a9-3139-00-00. */
+			s = strchr(info.path, '-');if (s) *s='\0';
 			CHECK (gp_port_info_list_append (list, info));
 		}
 		bus = bus->next;
@@ -224,6 +227,7 @@ gp_port_usb_open (GPPort *port)
 	int ret;
 	char name[64];
 
+	gp_log (GP_LOG_DEBUG,"libusb","gp_port_usb_open()");
 	if (!port || !port->pl->d)
 		return GP_ERROR_BAD_PARAMETERS;
 
@@ -253,10 +257,12 @@ gp_port_usb_open (GPPort *port)
 		if (ret < 0)
 			gp_port_set_error (port, _("Could not detach kernel driver '%s' of camera device."),name);
 	} else {
-		gp_port_set_error (port, _("Could not query kernel driver of device."));
+		if (errno != ENODATA) /* ENODATA - just no driver there */
+			gp_port_set_error (port, _("Could not query kernel driver of device."));
 	}
 #endif
 
+	gp_log (GP_LOG_DEBUG,"libusb","claiming interface %d", port->settings.usb.interface);
 	ret = usb_claim_interface (port->pl->dh,
 				   port->settings.usb.interface);
 	if (ret < 0) {
@@ -267,7 +273,6 @@ gp_port_usb_open (GPPort *port)
 			port->settings.usb.interface, "sdc2xx, stv680, spca50x");
 		return GP_ERROR_IO_USB_CLAIM;
 	}
-
 	return GP_OK;
 }
 
@@ -285,6 +290,16 @@ gp_port_usb_close (GPPort *port)
 		return (GP_ERROR_IO);
 	}
 
+	/* This is only for our very special Canon cameras which need a good
+	 * whack after close, otherwise they get timeouts on reconnect.
+	 */
+	if (port->pl->d->descriptor.idVendor == 0x04a9) {
+		if (usb_reset (port->pl->dh) < 0) {
+			gp_port_set_error (port, _("Could not reset USB port (%m)."));
+			return (GP_ERROR_IO);
+		}
+	}
+
 	if (usb_close (port->pl->dh) < 0) {
 		gp_port_set_error (port, _("Could not close USB port (%m)."));
 		return (GP_ERROR_IO);
@@ -361,11 +376,13 @@ gp_port_usb_check_int (GPPort *port, char *bytes, int size, int timeout)
 	if (!port || !port->pl->dh || timeout < 0)
 		return GP_ERROR_BAD_PARAMETERS;
 
-	ret = usb_bulk_read(port->pl->dh, port->settings.usb.intep,
+	ret = usb_interrupt_read(port->pl->dh, port->settings.usb.intep,
 			     bytes, size, timeout);
-        if (ret < 0)
+        if (ret < 0) {
+		if (errno == EAGAIN)
+			return GP_ERROR_TIMEOUT;
 		return GP_ERROR_IO_READ;
-
+	}
         return ret;
 }
 
@@ -423,6 +440,35 @@ gp_port_usb_msg_interface_read_lib(GPPort *port, int request,
 }
 
 
+/* The next two functions support the nonstandard request types 0x21 (write) 
+ * and 0xa1 (read), which are occasionally needed. 
+ */
+
+static int
+gp_port_usb_msg_class_write_lib(GPPort *port, int request, 
+	int value, int index, char *bytes, int size)
+{
+	if (!port || !port->pl->dh)
+		return GP_ERROR_BAD_PARAMETERS;
+
+	return usb_control_msg(port->pl->dh, 
+		USB_TYPE_CLASS | USB_RECIP_INTERFACE,
+		request, value, index, bytes, size, port->timeout);
+}
+
+
+static int
+gp_port_usb_msg_class_read_lib(GPPort *port, int request, 
+	int value, int index, char *bytes, int size)
+{
+	if (!port || !port->pl->dh)
+		return GP_ERROR_BAD_PARAMETERS;
+
+	return usb_control_msg(port->pl->dh, 
+		USB_TYPE_CLASS | USB_RECIP_INTERFACE | USB_ENDPOINT_IN,
+		request, value, index, bytes, size, port->timeout);
+}
+
 /*
  * This function applys changes to the device.
  *
@@ -438,9 +484,21 @@ gp_port_usb_update (GPPort *port)
 {
 	int ret, ifacereleased = FALSE;
 
+	gp_log (GP_LOG_DEBUG, "libusb", "gp_port_usb_update(old int=%d, conf=%d, alt=%d), (new int=%d, conf=%d, alt=%d)",
+		port->settings.usb.interface,
+		port->settings.usb.config,
+		port->settings.usb.altsetting,
+		port->settings_pending.usb.interface,
+		port->settings_pending.usb.config,
+		port->settings_pending.usb.altsetting
+	);
 	if (!port)
 		return GP_ERROR_BAD_PARAMETERS;
 
+	if (port->pl->interface == -1) port->pl->interface = port->settings.usb.interface;
+	if (port->pl->config == -1) port->pl->config = port->settings.usb.config;
+	if (port->pl->altsetting == -1) port->pl->altsetting = port->settings.usb.altsetting;
+
 	/* The portname can also be changed with the device still fully closed. */
 	memcpy(&port->settings.usb.port, &port->settings_pending.usb.port,
 		sizeof(port->settings.usb.port));
@@ -451,7 +509,32 @@ gp_port_usb_update (GPPort *port)
 	memcpy(&port->settings.usb, &port->settings_pending.usb,
 		sizeof(port->settings.usb));
 
+	/* The interface changed. release the old, claim the new ... */
+	if (port->settings.usb.interface != port->pl->interface) {
+		gp_log (GP_LOG_DEBUG, "libusb", "changing interface %d -> %d",
+			port->pl->interface,
+			port->settings.usb.interface
+		);
+		if (usb_release_interface (port->pl->dh,
+					   port->pl->interface) < 0) {
+			gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","releasing the iface for config failed.");
+			/* Not a hard error for now. -Marcus */
+		} else {
+			gp_log (GP_LOG_DEBUG,"libusb","claiming interface %d", port->settings.usb.interface);
+			ret = usb_claim_interface (port->pl->dh,
+						   port->settings.usb.interface);
+			if (ret < 0) {
+				gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","reclaiming the iface for config failed.");
+				return GP_ERROR_IO_UPDATE;
+			}
+			port->pl->interface = port->settings.usb.interface;
+		}
+	}
 	if (port->settings.usb.config != port->pl->config) {
+		gp_log (GP_LOG_DEBUG, "libusb", "changing config %d -> %d",
+			port->pl->config,
+			port->settings.usb.config
+		);
 		/* This can only be changed with the interface released. 
 		 * This is a hard requirement since 2.6.12.
 		 */
@@ -465,11 +548,14 @@ gp_port_usb_update (GPPort *port)
 		ret = usb_set_configuration(port->pl->dh,
 				     port->settings.usb.config);
 		if (ret < 0) {
+#if 0 /* setting the configuration failure is not fatal */
 			gp_port_set_error (port,
 				_("Could not set config %d/%d (%m)"),
 				port->settings.usb.interface,
 				port->settings.usb.config);
 			return GP_ERROR_IO_UPDATE;	
+#endif
+			gp_log (GP_LOG_ERROR, "gphoto2-port-usb","setting configuration from %d to %d failed with ret = %d, but continue...", port->pl->config, port->settings.usb.config, ret);
 		}
 
 		gp_log (GP_LOG_DEBUG, "gphoto2-port-usb",
@@ -478,6 +564,7 @@ gp_port_usb_update (GPPort *port)
 			port->settings.usb.config);
 
 		if (ifacereleased) {
+			gp_log (GP_LOG_DEBUG,"libusb","claiming interface %d", port->settings.usb.interface);
 			ret = usb_claim_interface (port->pl->dh,
 						   port->settings.usb.interface);
 			if (ret < 0) {
@@ -496,9 +583,9 @@ gp_port_usb_update (GPPort *port)
 		ret = usb_set_altinterface(port->pl->dh, port->settings.usb.altsetting);
 		if (ret < 0) {
 			gp_port_set_error (port, 
-				_("Could not set altsetting "
-				"%d/%d (%m)"),
-				port->settings.usb.interface,
+				_("Could not set altsetting from %d "
+				"to %d (%m)"),
+				port->pl->altsetting,
 				port->settings.usb.altsetting);
 			return GP_ERROR_IO_UPDATE;
 		}
@@ -598,7 +685,9 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 			continue;
 
 		for (dev = bus->devices; dev; dev = dev->next) {
-			if ((devname[0] != '\0') && strcmp(devname, dev->filename))
+			if (	(devname[0] != '\0') &&
+				(dev->filename != strstr(dev->filename, devname))
+			)
 				continue;
 
 			if ((dev->descriptor.idVendor == idvendor) &&
@@ -617,6 +706,8 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 
 				/* Set the defaults */
 				if (dev->config) {
+					int i;
+
 					if (dev->config[config].interface[interface].altsetting[altsetting].bInterfaceClass
 					    == USB_CLASS_MASS_STORAGE) {
 						gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
@@ -632,6 +723,15 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 					port->settings.usb.inep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_BULK);
 					port->settings.usb.outep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_OUT, USB_ENDPOINT_TYPE_BULK);
 					port->settings.usb.intep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_INTERRUPT);
+
+					port->settings.usb.maxpacketsize = 0;
+					gp_log (GP_LOG_DEBUG, "gphoto2-port-usb", "inep to look for is %02x", port->settings.usb.inep);
+					for (i=0;i<dev->config[config].interface[interface].altsetting[altsetting].bNumEndpoints;i++) {
+						if (port->settings.usb.inep == dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].bEndpointAddress) {
+							port->settings.usb.maxpacketsize = dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].wMaxPacketSize;
+							break;
+						}
+					}
 					gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
 						"Detected defaults: config %d, "
 						"interface %d, altsetting %d, "
@@ -846,6 +946,8 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 				class, subclass, protocol);
 			/* Set the defaults */
 			if (dev->config) {
+				int i;
+
 				port->settings.usb.config = dev->config[config].bConfigurationValue;
 				port->settings.usb.interface = dev->config[config].interface[interface].altsetting[altsetting].bInterfaceNumber;
 				port->settings.usb.altsetting = dev->config[config].interface[interface].altsetting[altsetting].bAlternateSetting;
@@ -853,6 +955,14 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 				port->settings.usb.inep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_BULK);
 				port->settings.usb.outep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_OUT, USB_ENDPOINT_TYPE_BULK);
 				port->settings.usb.intep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_INTERRUPT);
+				port->settings.usb.maxpacketsize = 0;
+				gp_log (GP_LOG_DEBUG, "gphoto2-port-usb", "inep to look for is %02x", port->settings.usb.inep);
+				for (i=0;i<dev->config[config].interface[interface].altsetting[altsetting].bNumEndpoints;i++) {
+					if (port->settings.usb.inep == dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].bEndpointAddress) {
+						port->settings.usb.maxpacketsize = dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].wMaxPacketSize;
+						break;
+					}
+				}
 				gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
 					"Detected defaults: config %d, "
 					"interface %d, altsetting %d, "
@@ -902,6 +1012,8 @@ gp_port_library_operations (void)
 	ops->msg_read   = gp_port_usb_msg_read_lib;
 	ops->msg_interface_write  = gp_port_usb_msg_interface_write_lib;
 	ops->msg_interface_read   = gp_port_usb_msg_interface_read_lib;
+	ops->msg_class_write  = gp_port_usb_msg_class_write_lib;
+	ops->msg_class_read   = gp_port_usb_msg_class_read_lib;
 	ops->find_device = gp_port_usb_find_device_lib;
 	ops->find_device_by_class = gp_port_usb_find_device_by_class_lib;
 
